# -*- coding: utf-8 -*-

import json
from typing import Dict, List, Any, Union

import pandas as pd

from .profile import BaseProfile
from .profile import CityProfile, NumClassProfile


class GenderProfile(BaseProfile):
    def __init__(self) -> None:
        super().__init__()
        self.dim = 2

    def convert(self, value):
        try:
            value = int(value)
        except Exception:
            return [0, 0]
        if value == 1:
            return [1, 0]
        elif value == 2:
            return [0, 1]
        else:
            return [0, 0]

    def inconvert(self, embedding):
        if embedding[0] == 1:
            return 'male'
        elif embedding[1] == 2:
            return 'feminine'
        else:
            return 'unknown_sex'


class NumScaleProfile(BaseProfile):
    def __init__(self, min_val=0, scale=1) -> None:
        super().__init__()
        self.dim = 1
        self.min_val = min_val
        self.scale = scale

    def convert(self, value):
        try:
            value = int(value)
        except Exception:
            return [0]
        
        value -= self.min_val
        value = min(value, self.scale)
        value = max(value, 0)

        return [value / self.scale]

    def inconvert(self, embedding: List[float]) -> str:
        if embedding[0] == 0:
            return 'unknown'
        else:
            return str(int(embedding[0] * self.scale))


class CertificationProfile(BaseProfile):
    def __init__(self) -> None:
        super().__init__()
        self.dim = 4
        # 这4类证书占比约95%
        self.certification_names = ['二级心理咨询师', '三级心理咨询师', '中级心理咨询师', '社会心理师']

    def convert(self, value):
        ret = [0, 0, 0, 0]
        try:
            index = self.certification_names.index(value)
            ret[index] = 1
        except ValueError:
            pass

        return ret

    def inconvert(self, embedding):
        for idx, val in enumerate(embedding):
            if val == 1:
                return self.certification_names[idx]
        
        return '其他证书'


class EducationProfile(BaseProfile):
    def __init__(self) -> None:
        super().__init__()
        self.dim = 6

    def convert(self, value):
        ret = [0, 0, 0, 0, 0, 0]

        try:
            value = int(value)
        except Exception:
            return ret

        if 3 <= value <= 8:
            ret[value - 3] = 1

        return ret

    def inconvert(self, embedding):
        for idx, val in enumerate(embedding):
            if val == 1:
                return str(idx + 3)
        
        return 'unknown_education'


class OrderCateProfile(BaseProfile):
    """
    order_tag_sum  订单标签类目
    """
    def __init__(self) -> None:
        super().__init__()
        self.dim = 10
        self.cate_list = ['恋爱情感', '情绪压力', '婚姻家庭', '个人成长', '亲子教育', '人际关系', '职场发展', '心理健康', '人际社交', '两性心理',]

    def convert(self, value):
        ret = [0] * self.dim

        try:
            value = json.loads('[' + value + ']')
        except Exception:
            return ret

        for item in value:
            try:
                i = self.cate_list.index(item['cate_name'])
                ret[i] = 1.0
            except:
                pass

        return ret

    def inconvert(self, embedding):
        ret = []
        for idx, val in enumerate(embedding):
            if val == 1.0:
                ret.append(self.cate_list[idx])
        
        return ret


class TagGoodAtDirectionProfile(BaseProfile):
    """
    tag_good_at_direction 咨询师擅长方向标签
    """
    def __init__(self) -> None:
        super().__init__()
        self.dim = 10
        self.cate_list = ['情绪压力', '亲子教育', '恋爱情感', '婚姻家庭', '心理健康', '个人成长', '人际关系', '职场发展', '人际社交', '两性心理',]

    def convert(self, value):
        ret = [0] * self.dim

        try:
            value = json.loads(value)
        except Exception:
            return ret

        for item in value:
            try:
                i = self.cate_list.index(item['cate_name'])
                ret[i] = 1.0
            except:
                pass

        return ret

    def inconvert(self, embedding):
        ret = []
        for idx, val in enumerate(embedding):
            if val == 1.0:
                ret.append(self.cate_list[idx])
        
        return ret


class CateGoodAtProfile(BaseProfile):
    """
    cate_good_at 擅长标签
    """
    def __init__(self) -> None:
        super().__init__()
        self.dim = 8
        self.cate_list = ['婚姻家庭', '人际关系', '个人成长', '情绪压力', '恋爱情感', '亲子教育', '心理健康', '职场发展']

    def convert(self, value):
        ret = [0] * self.dim

        try:
            value = json.loads(value)
        except Exception:
            return ret

        for item in value:
            try:
                i = self.cate_list.index(item['cate_name'])
                ret[i] = min(item['sort_score'], 60) / 60
            except:
                pass

        return ret

    def inconvert(self, embedding):
        ret = []
        for idx, val in enumerate(embedding):
            ret.append({
                'cate_name': self.cate_list[idx],
                'sort_score': val,
            })
    
        return ret


class TagGoodAtSubclassProfile(BaseProfile):
    """
    tag_good_at_** 咨询师擅长标签，包括：
        咨询师擅长群体标签
        咨询师咨询风格标签
        咨询师擅长服务标签
        咨询师语言能力标签
        咨询师咨询流派标签
    """
    def __init__(self, cate_list) -> None:
        super().__init__()
        self.dim = len(cate_list)
        self.cate_list = cate_list

    def convert(self, value):
        ret = [0] * self.dim

        try:
            value = json.loads(value)
        except Exception:
            return ret

        for item in value:
            try:
                i = self.cate_list.index(item['tag_name'])
                ret[i] = 1.0
            except:
                pass

        return ret

    def inconvert(self, embedding):
        ret = []
        for idx, val in enumerate(embedding):
            if val == 1.0:
                ret.append(self.cate_list[idx])
        
        return ret


good_at_group_tag_list = [
    '青少年', '职场人', '家庭关系', '大学生', '伴侣', '夫妻', '中学生', '公务员', '留学生',
    '精神康复者', '小学生', '孕产妇', '老年人', '成年人', '中年人',
]
good_at_style_tag_list = [
    '温暖', '温和而真诚', '耐心', '专业', '抱持', '赋能', '温和而坚定', '深刻而有力', '沉稳',
    '真实', '深入', '富于创造性', '清晰', '简洁', '接纳包容', '真诚', '敏锐专业', '温和', '坚定',
]
good_at_service_tag_list = [
    '情绪疏导', '关系改善', '情绪管理', '情感分析', '情感陪伴', '心理分析', '动力提升', '创伤修复',
    '恋爱指导', '行为矫正', '离婚指导', '认知调整', '心态调整', '经验指导', '身心减压', '性格改善',
    '失眠改善', '绘画分析', '情感修复', '梦境解析', '习惯养成', '危机消除',
]
good_at_language_tag_list = [
    '普通话', '英语', '粤语', '四川话', '江西方言', '闽南语', '日语', '韩语', '德语', '俄语',
    '西班牙语', '法语', '意大利语',
]
good_at_school_tag_list = [
    '认知行为疗法', '叙事疗法', '整合疗法', '精神分析', '焦点解决短期疗法', '人本主义治疗',
    '婚姻与家庭治疗', '催眠疗法', '森田疗法', '格式塔疗法', '心理动力学疗法', '沙盘治疗', '其他疗法',
    '表达性艺术疗法', '接纳承诺疗法', '心理危机干预', '辩证行为疗法', '游戏治疗', '情感关注疗法',
]

profile_converters = [
    ['gender', GenderProfile()],
    ['age', NumScaleProfile(0, 100)],
    ['certification', CertificationProfile()],
    ['education', EducationProfile()],
    ['city', CityProfile(level=4)],
    ['work_years', NumScaleProfile(0, 20)],
    ['product_min_price', NumScaleProfile(10, 1200-10)],
    ['order_cate_sum', OrderCateProfile()],
    ['tag_good_at_direction', TagGoodAtDirectionProfile()],
    ['tag_good_at_group', TagGoodAtSubclassProfile(good_at_group_tag_list)],
    ['tag_good_at_style', TagGoodAtSubclassProfile(good_at_style_tag_list)],
    ['tag_good_at_service', TagGoodAtSubclassProfile(good_at_service_tag_list)],
    ['tag_good_at_language', TagGoodAtSubclassProfile(good_at_language_tag_list)],
    ['tag_good_at_school', TagGoodAtSubclassProfile(good_at_school_tag_list)],
    ['cate_good_at', CateGoodAtProfile()],
]

def encode_counselor_profile(profile):
    """
    将咨询师画像信息转换为向量
    """
    embedding = []
    for [name, converter] in profile_converters:
        embedding.extend(converter.convert(profile[name]))
    return embedding


def decode_counselor_profile(embedding):
    """
    向量转换为咨询师画像
    """
    ret = {}
    si = 0
    for [name, converter] in profile_converters:
        ei = si + converter.dim
        ret[name] = converter.inconvert(embedding[si: ei])
        si = ei
    return ret